searchNodes=[{"doc":"Rationalize can find a close rational approximation to any decimal number. This library also provides a small, incomplete tool set for working with rational numbers. Basic usage Find the closest rational approximation to 0.17 with a denominator no larger than 10: &gt; Rationalize . closest_rational ( 0.17 , 10 ) % Rationalize.Rational { n : 1 , d : 6 } Notice that the function closest_rational returns a struct with fields n and d that hold the numerator and denominator, respectively. A rational number can be converted to a float or a string: &gt; Rationalize . to_float ( % Rationalize.Rational { n : 1 , d : 6 } ) { :ok , 0.16666666666666666 } &gt; Rationalize . to_string ( % Rationalize.Rational { n : 1 , d : 6 } ) &quot;1/6&quot; The precision of the rational approximation can be increased by increasing the maximum denominator: &gt; Rationalize . closest_rational ( 0.17 , 100 ) % Rationalize.Rational { n : 17 , d : 100 } This last result is exact. Further increase in the maximum denominator will not change the result: &gt; Rationalize . closest_rational ( 0.17 , 1000 ) % Rationalize.Rational { n : 17 , d : 100 } Advanced usage Working with rational numbers safely means avoiding conversion to floating numbers unless it's necessary, properly handling cases where the numerator or denominator may be 0, and allowing the numerator or denominator to be negative. The module Rationalize.Rational provides a small set of safe operations: # Compare two rational numbers &gt; r1 = Rationalize.Rational . new ( 1 , 0 ) &gt; r2 = Rationalize.Rational . new ( 1 , 2 ) &gt; Rationalize.Rational . compare ( r1 , r2 ) :gt # r1 &gt; r2 # Compute the numerical difference between a rational and a float &gt; r3 = Rationalize.Rational . new ( 3 , 2 ) &gt; Rationalize.Rational . diff_num ( r3 , 0.5 ) { :ok , 1.0 } # Operations with negative rationals are allows &gt; r4 = Rationalize.Rational . new ( - 3 , 2 ) &gt; Rationalize.Rational . diff_num ( r4 , 0.5 ) { :ok , - 2.0 } # A difference can return :pos_infinity or :neg_infinity &gt; r5 = Rationalize.Rational . new ( - 1 , 0 ) &gt; Rationalize.Rational . diff_num ( 0.5 , r5 ) :pos_infinity # Any operations with 0/0 return :undefined &gt; r6 = Rationalize.Rational . new ( 0 , 0 ) &gt; Rationalize.Rational . diff_num ( r6 , 1 ) :undefined The utilities in Rationalize.Rational were originally built only to support the search for rational approximations. But they have general applicability, and the tool set could be expanded in the future. Use cases Rationalize was originally developed for two very specific use cases. Probabilities as fractions When a probability is presented in decimal form, it may be challenging for some readers to create a mental model of what that probability might mean . For example, a decimal probability could be presented like this in a short narrative: The chance of a catastrophic meltdown is 0.17. But it might be preferable to write this: The chance of a catastrophic meltdown is 1 in 6. Rational approximations It's sometimes computationally convenient to have a rational approximation of a decimal number. With a number like pi = 3.14159265359... it might be handy to remember that 22/7 is a reasonably close approximation (about 3.143). How does it work? Rationalize uses a nice bit of math to find close rational approximations. The Stern-Brocot tree is a binary tree that holds all positive rational numbers whose numerator and denominator are relatively prime (no common factors). To find a rational approximation to a decimal number, a binary search is used to navigate the tree until some termination condition is satisfied. Each update of the binary search uses the mediant of two rational numbers. If R1 = n1/d1 and R2 = n2/d2 are two rational numbers, and R1 &lt; R2 , the mediant is M = (n1 + n2) / (d1 + d2) , which has the property R1 &lt; M &lt; R2 . For a target decimal d and R1 &lt; d &lt; R2 , the mediant is used to recursively narrow the range of the bracketing rationals. Because of the properties of the Stern-Brocot tree and the mediant-based search, the rational returned by closest_rational will always have relatively prime numerator and denominator. Todo Rationalize grew from a specific use case. But... Adaptively compute the maximum denominator based on the size of the decimal. Add more utilities for working with rational numbers (e.g., add, multiply, divide, subtract), and bring these into the top-level API.","ref":"Rationalize.html","title":"Rationalize","type":"module"},{"doc":"See Rationalize.Search.closest_rational/2 .","ref":"Rationalize.html#closest_rational/2","title":"Rationalize.closest_rational/2","type":"function"},{"doc":"See Rationalize.Rational.to_float/1 .","ref":"Rationalize.html#to_float/1","title":"Rationalize.to_float/1","type":"function"},{"doc":"See Rationalize.Rational.to_string/1 .","ref":"Rationalize.html#to_string/1","title":"Rationalize.to_string/1","type":"function"},{"doc":"See Rationalize.Rational.to_string/2 .","ref":"Rationalize.html#to_string/2","title":"Rationalize.to_string/2","type":"function"},{"doc":"Construct and safely manipulate rational numbers. A rational number is defined by its integer numerator and denominator, each of which may be positive, negative, or zero. A denominator of 0 is perfectly acceptable for a rational number as long floating point division is avoided.","ref":"Rationalize.Rational.html","title":"Rationalize.Rational","type":"module"},{"doc":"Compare two rational numbers. The result is :gt , :lt , or :eq if the first argument is, respectively, greater than, less than, or equal to the second argument. The comparison is done in a way that avoids conversion to a finite-precision float. The result is :undefined if one of the arguments has numerator and denominator equal to zero, or if both arguments have a denominator of 0. A rational with a denominator of 0 and a positive numerator is greater than any rational with a nonzero denominator. Similarly, a rational with a denominator of 0 and a negative numerator is less than any rational with a nonzero denominator. Examples iex&gt; alias Rationalize.Rational, as: R iex&gt; R.compare(R.new(1, 2), R.new(2, 1)) :lt iex&gt; R.compare(R.new(2, 1), R.new(1, 2)) :gt iex&gt; R.compare(R.new(1, 1), R.new(1, 1)) :eq iex&gt; R.compare(R.new(1, 0), R.new(1, 2)) :gt iex&gt; R.compare(R.new(-1, 0), R.new(-1, 2)) :lt Notes While shortcuts may be possible, the explicit branches are easy to follow.","ref":"Rationalize.Rational.html#compare/2","title":"Rationalize.Rational.compare/2","type":"function"},{"doc":"Compare a rational number to an ordinary number (integer or float). The result is :gt , :lt , or :eq if the rational is, respectively, greater than, less than, or equal to the float. The result is :undefined if the numerator and denominator of the rational are both 0. A rational with a denominator of 0 and a positive numerator is greater than any number. Similarly, a rational with a denominator of 0 and a negative numerator is less than any number. Examples iex&gt; alias Rationalize.Rational, as: R iex&gt; R.compare_to_num(R.new(1, 2), 1) :lt","ref":"Rationalize.Rational.html#compare_to_num/2","title":"Rationalize.Rational.compare_to_num/2","type":"function"},{"doc":"Attempt to compute the difference between a rational number and a float. Arguments can be supplied in either order. This returns {:ok, &lt;float diff&gt;} when the rational can be converted to a finite float. If the denominator of the rational is 0, this returns :undefined if the numerator is also 0, :pos_inifinity if the numerator is greater than 0, or :neg_infinity if the numerator is less than 0.","ref":"Rationalize.Rational.html#diff_num/2","title":"Rationalize.Rational.diff_num/2","type":"function"},{"doc":"Return the mediant of two rational numbers. If n1/d1 and n2/d2 are two rational numbers, then mediant(n1/d1, n2/d2) = (n1 + n2)/(d1 + d2) The mediant has the property n1/d1 &lt; (n1 + n2)/(d1 + d2) &lt; n2/d2 when n1/d1 &lt; n2/d2 . Examples iex&gt; alias Rationalize.Rational, as: R iex&gt; R.mediant(R.new(1, 4), R.new(1, 2)) %R{n: 2, d: 6}","ref":"Rationalize.Rational.html#mediant/2","title":"Rationalize.Rational.mediant/2","type":"function"},{"doc":"Negate a rational by negating the numerator. Note that this does the right thing when the denominator is 0.","ref":"Rationalize.Rational.html#negate/1","title":"Rationalize.Rational.negate/1","type":"function"},{"doc":"Create a new Rational struct given integer values for the numerator and denominator. An exception is raised if the numerator or denominator is not an integer.","ref":"Rationalize.Rational.html#new/2","title":"Rationalize.Rational.new/2","type":"function"},{"doc":"Standardize a rational number such that the denominator is never negative. The result is numerically equivalent to the original rational, but not syntactically equivalent. Examples iex&gt; alias Rationalize.Rational, as: R iex&gt; R.standardize(R.new(1, 2)) %R{n: 1, d: 2} iex&gt; R.standardize(R.new(1, -2)) %R{n: -1, d: 2}","ref":"Rationalize.Rational.html#standardize/1","title":"Rationalize.Rational.standardize/1","type":"function"},{"doc":"Attempt to convert a rational number to a float. This returns {:ok, &lt;float value&gt;} for the given rational number if the denominator is nonzero. If the denominator is 0, this returns :undefined if the numerator is also 0, :pos_inifinity if the numerator is greater than 0, or :neg_infinity if the numerator is less than 0.","ref":"Rationalize.Rational.html#to_float/1","title":"Rationalize.Rational.to_float/1","type":"function"},{"doc":"Return a string representation of a rational number using a provided format. The default format is &quot;&lt;numerator&gt;/&lt;denominator&gt;&quot;, but this can be customized by providing a function that takes the numerator and denominator and returns a string. Examples iex&gt; alias Rationalize.Rational, as: R iex&gt; R.to_string(R.new(1, 2)) &quot;1/2&quot; iex&gt; R.to_string(R.new(1, 2), fn n, d -&gt; &quot;\#{n} over \#{d}&quot; end) &quot;1 over 2&quot;","ref":"Rationalize.Rational.html#to_string/2","title":"Rationalize.Rational.to_string/2","type":"function"},{"doc":"","ref":"Rationalize.Rational.html#t:comparison/0","title":"Rationalize.Rational.comparison/0","type":"type"},{"doc":"","ref":"Rationalize.Rational.html#t:option_float/0","title":"Rationalize.Rational.option_float/0","type":"type"},{"doc":"","ref":"Rationalize.Rational.html#t:t/0","title":"Rationalize.Rational.t/0","type":"type"},{"doc":"Implementation of the search strategy for neaby rational numbers that bracket a given number. The search for the bracketing rationals involves a traversal of the Stern-Brocot tree. Since the Stern-Brocot tree is essentially a binary tree, the search amounts to a binary search. Every node in the Stern-Brocot tree is a rational number with relatively prime numerator and denominator (no common factors). It follows that each rational that brackets num has relatively prime components and cannot be further reduced.","ref":"Rationalize.Search.html","title":"Rationalize.Search","type":"module"},{"doc":"Return the bracket of rational numbers {r1, r2} that most closely bounds the number num , such that both rationals have denominators no larger than integer den_max . Examples iex&gt; Rationalize.Search . closest_bracket ( 0.27 , 10 ) { % Rationalize.Rational { n : 1 , d : 4 } , % Rationalize.Rational { n : 2 , d : 7 } }","ref":"Rationalize.Search.html#closest_bracket/2","title":"Rationalize.Search.closest_bracket/2","type":"function"},{"doc":"Return the rational fraction closest to the number num with denominator no larger than integer den_max . Examples iex&gt; Rationalize . closest_rational ( 0.27 , 10 ) % Rationalize.Rational { n : 2 , d : 7 }","ref":"Rationalize.Search.html#closest_rational/2","title":"Rationalize.Search.closest_rational/2","type":"function"},{"doc":"Recursively narrow the rational bracket around number num until one of the termination conditions is satisfied. The search terminates when: either end of the input bracket numerically equals num . the denominator of the mediant of the input bracket exceeds den_max . the mediant of the input bracket equals num .","ref":"Rationalize.Search.html#search/3","title":"Rationalize.Search.search/3","type":"function"},{"doc":"","ref":"Rationalize.Search.html#t:bracket/0","title":"Rationalize.Search.bracket/0","type":"type"},{"doc":"Rationalize can find a close rational approximation to any decimal number. This library also provides a small, incomplete tool set for working with rational numbers. Basic usage Find the closest rational approximation to 0.17 with a denominator no larger than 10: &gt; Rationalize . closest_rational ( 0.17 , 10 ) % Rationalize.Rational { n : 1 , d : 6 } Notice that the function closest_rational returns a struct with fields n and d that hold the numerator and denominator, respectively. A rational number can be converted to a float or a string: &gt; Rationalize . to_float ( % Rationalize.Rational { n : 1 , d : 6 } ) { :ok , 0.16666666666666666 } &gt; Rationalize . to_string ( % Rationalize.Rational { n : 1 , d : 6 } ) &quot;1/6&quot; The precision of the rational approximation can be increased by increasing the maximum denominator: &gt; Rationalize . closest_rational ( 0.17 , 100 ) % Rationalize.Rational { n : 17 , d : 100 } This last result is exact. Further increase in the maximum denominator will not change the result: &gt; Rationalize . closest_rational ( 0.17 , 1000 ) % Rationalize.Rational { n : 17 , d : 100 } Advanced usage Working with rational numbers safely means avoiding conversion to floating numbers unless it's necessary, properly handling cases where the numerator or denominator may be 0, and allowing the numerator or denominator to be negative. The module Rationalize.Rational provides a small set of safe operations: # Compare two rational numbers &gt; r1 = Rationalize.Rational . new ( 1 , 0 ) &gt; r2 = Rationalize.Rational . new ( 1 , 2 ) &gt; Rationalize.Rational . compare ( r1 , r2 ) :gt # r1 &gt; r2 # Compute the numerical difference between a rational and a float &gt; r3 = Rationalize.Rational . new ( 3 , 2 ) &gt; Rationalize.Rational . diff_num ( r3 , 0.5 ) { :ok , 1.0 } # Operations with negative rationals are allows &gt; r4 = Rationalize.Rational . new ( - 3 , 2 ) &gt; Rationalize.Rational . diff_num ( r4 , 0.5 ) { :ok , - 2.0 } # A difference can return :pos_infinity or :neg_infinity &gt; r5 = Rationalize.Rational . new ( - 1 , 0 ) &gt; Rationalize.Rational . diff_num ( 0.5 , r5 ) :pos_infinity # Any operations with 0/0 return :undefined &gt; r6 = Rationalize.Rational . new ( 0 , 0 ) &gt; Rationalize.Rational . diff_num ( r6 , 1 ) :undefined The utilities in Rationalize.Rational were originally built only to support the search for rational approximations. But they have general applicability, and the tool set could be expanded in the future. Use cases Rationalize was originally developed for two very specific use cases.","ref":"readme.html","title":"README","type":"extras"},{"doc":"When a probability is presented in decimal form, it may be challenging for some readers to create a mental model of what that probability might mean . For example, a decimal probability could be presented like this in a short narrative: The chance of a catastrophic meltdown is 0.17. But it might be preferable to write this: The chance of a catastrophic meltdown is 1 in 6.","ref":"readme.html#probabilities-as-fractions","title":"README - Probabilities as fractions","type":"extras"},{"doc":"It's sometimes computationally convenient to have a rational approximation of a decimal number. With a number like pi = 3.14159265359... it might be handy to remember that 22/7 is a reasonably close approximation (about 3.143). How does it work? Rationalize uses a nice bit of math to find close rational approximations. The Stern-Brocot tree is a binary tree that holds all positive rational numbers whose numerator and denominator are relatively prime (no common factors). To find a rational approximation to a decimal number, a binary search is used to navigate the tree until some termination condition is satisfied. Each update of the binary search uses the mediant of two rational numbers. If R1 = n1/d1 and R2 = n2/d2 are two rational numbers, and R1 &lt; R2 , the mediant is M = (n1 + n2) / (d1 + d2) , which has the property R1 &lt; M &lt; R2 . For a target decimal d and R1 &lt; d &lt; R2 , the mediant is used to recursively narrow the range of the bracketing rationals. Because of the properties of the Stern-Brocot tree and the mediant-based search, the rational returned by closest_rational will always have relatively prime numerator and denominator. Todo Rationalize grew from a specific use case, but it could be generalized to be a more complete library for working with rational numbers. Possibilities for extending Rationalize include adaptively computing the maximum denominator based on the size of the decimal. adding more utilities for working with rational numbers (e.g., add, multiply, divide, subtract), and bringing these into the top-level API.","ref":"readme.html#rational-approximations","title":"README - Rational approximations","type":"extras"}]